import { TQSocketProtocolPayloadData, TQSocketProtocolCompressor, IQSocketProtocolMessage, IQSocketProtocolMessageMetaAck, QSocketProtocol, IQSocketProtocolPayload, IQSocketProtocolMessageMetaData, IQSocketProtocolMessageMetaControl, IQSocketProtocolChunk } from '@qsocket/protocol';
import { IQSocketTransportEIOLatestServer, IQSocketTransportEIOV3Server, IQSocketTransportEIOClientLatestSocket, IQSocketTransportEIOClientV3Socket, IQSocketTransportEIOLatestSocket, IQSocketTransportEIOV3Socket } from '@qsocket/transport';

type TQSocketContentType = 'undefined' | 'null' | 'boolean' | 'number' | 'string' | 'json' | 'buffer';
type TQSocketContentEncoding = 'raw' | 'gzip' | 'deflate';
type TListennerReturn<T extends TQSocketProtocolPayloadData> = Promise<T> | T;
type TQSocketListenerCallback<I extends TQSocketProtocolPayloadData, O extends TQSocketProtocolPayloadData> = (payload: I, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding) => TListennerReturn<O>;

interface IQSocketProtocolConfig {
    compressor?: {
        on: boolean;
        compressor?: TQSocketProtocolCompressor;
        compressionFromSize?: number;
    };
}
interface IQSocketLogger {
    log(...message: any[]): void;
    error(...message: any[]): void;
    info(...message: any[]): void;
    debug(...message: any[]): void;
    warn(...message: any[]): void;
}
interface IQSocketDebugConfig {
    /** Включить отладку */
    on: boolean;
    /** Инстанс логгера */
    logger?: IQSocketLogger;
    /** Префикс перед всеми записями */
    prefix?: string;
}

declare class QSocketDebuger {
    /** Включен ли режим отладки */
    private debug;
    /** Префикс для сообщений логирования */
    private prefix;
    /** Логгер, используемый для вывода сообщений */
    private logger;
    /**
     * Конструктор класса QSocketUtils.
     * @param {IQSocketDebugConfig} [debugConfig] - Конфигурация для режима отладки.
     */
    constructor(debugConfig?: IQSocketDebugConfig);
    /**
     * Логирует сообщение, если включен режим отладки.
     * @param {...any[]} message - Сообщение или данные для логирования.
     */
    log(...message: any[]): void;
    /**
     * Логирует сообщение об ошибке, если включен режим отладки.
     * @param {...any[]} message - Сообщение или данные для логирования ошибок.
     */
    error(...message: any[]): void;
    /**
     * Логирует информационное сообщение, если включен режим отладки.
     * @param {...any[]} message - Сообщение или данные для информационного логирования.
     */
    info(...message: any[]): void;
    /**
     * Логирует предупреждение, если включен режим отладки.
     * @param {...any[]} message - Сообщение или данные для логирования предупреждений.
     */
    warn(...message: any[]): void;
}

type TQSocketServer = IQSocketTransportEIOLatestServer | IQSocketTransportEIOV3Server;
type TQSocketClientSocket = IQSocketTransportEIOClientLatestSocket | IQSocketTransportEIOClientV3Socket;
type TQSocketServerSocket = IQSocketTransportEIOLatestSocket | IQSocketTransportEIOV3Socket;
type TQSocketInteractionInstance = TQSocketClientSocket | TQSocketServerSocket;

declare class QSocketClient extends QSocketBase {
    constructor(socket: TQSocketClientSocket, protocolConfig?: IQSocketProtocolConfig, debugConfig?: IQSocketDebugConfig);
    namespaceHandle(namespace: QSocketNamespace): void;
}

/**
 * Класс для генерации уникальных идентификаторов (UUID), с возможностью добавления префикса.
 */
declare class QSocketUniqueGenerator<T extends string = ''> {
    /**
     * Текущий индекс для генерации UUID.
     * @private
     */
    private uuidIndex;
    private prefix;
    constructor(prefix?: T);
    /**
     * Метод для генерации следующего уникального идентификатора.
     */
    next(): `${T}${string}`;
}

/**
 * Абстрактный базовый класс для работы с QSocket.
 * Предоставляет методы для управления пространствами имён.
 */
declare abstract class QSocketBase {
    readonly type: 'client' | 'server';
    readonly id: `${'C' | 'S'}${string}`;
    protected readonly uuid: QSocketUniqueGenerator<`${'C' | 'S'}${string}-SM`>;
    protected readonly interactionUUID: QSocketUniqueGenerator<`${'C' | 'S'}${string}-I${string}`>;
    protected readonly namespaces: Map<string, QSocketNamespace>;
    protected readonly debuger: QSocketDebuger;
    protected readonly interactions: Map<`${'C' | 'S'}${string}-I${string}`, QSocketInteraction>;
    protected readonly protocolConfig: IQSocketProtocolConfig;
    /**
     * Конструктор класса QSocketBase.
     * @param {IQSocketDebugConfig} [debugConfig] - Конфигурация для режима отладки.
     */
    constructor(type: 'client' | 'server', protocolConfig?: IQSocketProtocolConfig, debugConfig?: IQSocketDebugConfig);
    protected connectionHandle(socket: TQSocketServerSocket | TQSocketClientSocket, server: QSocketServer | QSocketClient): void;
    /**
     * Создаёт новое пространство имён или возвращает существующее.
     * @param {string} name - Имя создаваемого пространства имён.
     * @returns {QSocketNamespace} Пространство имён QSocket.
     */
    createNamespace(name: string): QSocketNamespace;
    /**
     * Удаляет существующее пространство имён.
     * @param {string} name - Имя удаляемого пространства имён.
     * @returns {boolean} Возвращает `true`, если пространство имён было удалено, иначе `false`.
     */
    removeNamespace(name: string): boolean;
    /**
     * Получает пространство имён по его имени.
     * @param {string} name - Имя запрашиваемого пространства имён.
     * @returns {QSocketNamespace | undefined} Пространство имён или `undefined`, если оно не найдено.
     */
    getNamespace(name: string): QSocketNamespace | undefined;
    /**
     * Абстрактный метод для обработки логики после создания пространства имён.
     * @param {QSocketNamespace} namespace - Пространство имён, требующее обработки.
     */
    protected abstract namespaceHandle(namespace: QSocketNamespace): void;
}

declare class QSocketServer extends QSocketBase {
    private server;
    constructor(transport: TQSocketServer, protocolConfig: IQSocketProtocolConfig, debugConfig?: IQSocketDebugConfig);
    protected namespaceHandle(): void;
}

declare class QSocketInteraction {
    readonly uuid: QSocketUniqueGenerator<`${'S' | 'C'}${string}-I${string}-M`>;
    readonly id: `${'S' | 'C'}${string}-I${string}`;
    server: QSocketServer | QSocketClient;
    socket: TQSocketInteractionInstance;
    acks: Map<string, (ackResult: IQSocketProtocolMessage<IQSocketProtocolMessageMetaAck>[number]['payload'][]) => void>;
    namespaces: Map<string, QSocketNamespace>;
    protocol: QSocketProtocol;
    debuger: QSocketDebuger;
    interactions: Map<`${'C' | 'S'}${string}-I${string}`, QSocketInteraction>;
    private middlewares;
    constructor(id: `${'S' | 'C'}${string}-I${string}`, socket: TQSocketInteractionInstance, server: QSocketServer | QSocketClient, interactions: Map<`${'C' | 'S'}${string}-I${string}`, QSocketInteraction>, protocolConfig: IQSocketProtocolConfig, debuger: QSocketDebuger);
    static addNamespace(interaction: QSocketInteraction, namespace: QSocketNamespace): void;
    private protocolConfigure;
    /** Главный обработчик сообщений */
    private onHandle;
    private onData;
    private onAck;
    private onControl;
    sendData<O extends IQSocketProtocolPayload = IQSocketProtocolPayload>(message: IQSocketProtocolMessage<IQSocketProtocolMessageMetaData>): Promise<O[][] | undefined>;
    broadcast<O extends IQSocketProtocolPayload = IQSocketProtocolPayload>(message: IQSocketProtocolMessage<IQSocketProtocolMessageMetaData>): Promise<O[][][]>;
    sendAck(message: IQSocketProtocolMessage<IQSocketProtocolMessageMetaAck>): Promise<void>;
    sendCommand(message: IQSocketProtocolMessage<IQSocketProtocolMessageMetaControl>): Promise<IQSocketProtocolMessage<IQSocketProtocolMessageMetaAck>[number]['payload'][][] | void>;
    /**
     * @description Добавляет промежуточный обработчик сообщений
     * @param handler
     */
    use(handler: (message: IQSocketProtocolMessage<IQSocketProtocolMessageMetaData>, socket: TQSocketInteractionInstance) => IQSocketProtocolMessage<IQSocketProtocolMessageMetaData>): void;
}

/**
 * Класс `QSocketNamespace` представляет пространство имён для управления подключениями и событиями клиентов.
 */
declare class QSocketNamespace {
    /**
     * @private
     * @readonly
     * @type {string}
     * @description Уникальное название пространства имён.
     */
    private readonly _name;
    /**
     * @private
     * @type {((connection: QSocketConnection) => void)[]}
     * @description Список слушателей для событий подключения клиента к пространству имён.
     */
    private connectionListeners;
    /**
     * @private
     * @type {Map<QSocketInteraction, QSocketConnection>}
     * @description Карта активных соединений, связывающая клиентов с их подключениями.
     */
    private connections;
    /**
     * @public
     * @type {string}
     * @description Возвращает название пространства имён.
     */
    get name(): string;
    private debuger;
    private middlewares;
    /**
     * @constructor
     * @param {string} name - Название нового пространства имён.
     * @throws {Error} Если пространство имён с таким именем уже существует.
     * @description Создаёт новое пространство имён и добавляет его имя в список уникальных.
     */
    constructor(name: string, debuger: QSocketDebuger);
    /**
     * @public
     * @param {'connection'} event - Событие, которое требуется отслеживать. В данный момент поддерживается только 'connection'.
     * @param {(connection: QSocketConnection) => void} listener - Слушатель, который будет вызван при наступлении события.
     * @throws {Error} Если событие не поддерживается.
     * @description Добавляет слушателя на событие подключения нового клиента к пространству имён.
     */
    on(event: 'connection', listener: (connection: QSocketConnection) => void): void;
    /**
     * @public
     * @static
     * @param {QSocketNamespace} namespace - Экземпляр пространства имён, из которого будет удалён клиент.
     * @param {IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>} message - Сообщение по протоколу Q-SOCKET-DATA.
     * @description Организует поток данных от клиента к активным соединениям передаваемого пространства имён.
     */
    static pipe(interaction: QSocketInteraction, namespace: QSocketNamespace, message: IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>): Promise<IQSocketProtocolMessage<IQSocketProtocolMessageMetaAck>>;
    /**
     * @private
     * @param {IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>} message - Сообщение по протоколу Q-SOCKET-DATA.
     * @description Организует поток данных от клиента к активным соединениям текущего пространства имён.
     */
    private $__pipe;
    /**
     * @public
     * @static
     * @param {QSocketNamespace} namespace - Экземпляр пространства имён, к которому добавляется клиент.
     * @param {QSocketInteraction} interaction - Экземпляр клиента, который будет добавлен.
     * @description Добавляет клиента к указанному пространству имён, вызывая метод экземпляра для подключения.
     */
    static addClient(namespace: QSocketNamespace, interaction: QSocketInteraction): void;
    /**
     * @private
     * @param {QSocketInteraction} interaction - Экземпляр клиента для добавления.
     * @description Создаёт соединение для клиента, добавляет его в карту подключений и вызывает всех слушателей события "connection".
     */
    private $__addClient;
    /**
     * @public
     * @static
     * @param {QSocketNamespace} namespace - Экземпляр пространства имён, из которого будет удалён клиент.
     * @param {QSocketInteraction} interaction - Экземпляр клиента, который будет удалён.
     * @description Удаляет клиента из указанного пространства имён, вызывая метод экземпляра для отключения.
     */
    static deleteClient(namespace: QSocketNamespace, interaction: QSocketInteraction): void;
    /**
     * @private
     * @param {QSocketInteraction} interaction - Экземпляр клиента для удаления.
     * @description Удаляет соединение клиента из карты подключений и закрывает его.
     */
    private $__deleteClient;
    /**
     * @public
     * @description Уничтожает текущее пространство имён: удаляет все активные соединения клиентов и очищает его из списка уникальных имён.
     */
    destroy(): void;
    emit<I extends TQSocketProtocolPayloadData, O extends IQSocketProtocolPayload>(event: string, data: I, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding): Promise<O[][]>;
    /**
     * @description Добавляет промежуточный обработчик чанков сообщения
     * @param handler
     */
    use(handler: (chunk: IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>) => IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>): void;
}

declare class QSocketConnection {
    private interaction;
    private namespace;
    private events;
    private closeListeners;
    constructor(interaction: QSocketInteraction, namespace: QSocketNamespace);
    /**
     * @description Подписка на событие завершения соединения с клиентом
     */
    onClose(listener: () => void): void;
    /**
     * @description Закрытие соединения
     */
    private $__close;
    static close(connection: QSocketConnection): void;
    /**
     * @description Подписка на событие с поддержкой нескольких слушателей
     */
    on<I extends TQSocketProtocolPayloadData, O extends TQSocketProtocolPayloadData>(event: string, listener: TQSocketListenerCallback<I, O>, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding): void;
    /**
     * @description Подписка на одноразовое событие
     */
    once<I extends TQSocketProtocolPayloadData, O extends TQSocketProtocolPayloadData>(event: string, listener: TQSocketListenerCallback<I, O>, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding): void;
    /**
     * @description Производит отписку от события
     */
    off<I extends TQSocketProtocolPayloadData, O extends TQSocketProtocolPayloadData>(event: string, listener: TQSocketListenerCallback<I, O>): false | undefined;
    /**
     * @description Отправка данных на связанный клиент
     */
    emit<I extends TQSocketProtocolPayloadData, O extends IQSocketProtocolPayload>(event: string, data: I, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding): Promise<O[]>;
    broadcast<I extends TQSocketProtocolPayloadData, O extends IQSocketProtocolPayload>(event: string, data: I, contentType?: TQSocketContentType, contentEncoding?: TQSocketContentEncoding): Promise<O[][] | undefined>;
    static pipe(connection: QSocketConnection, message: IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>): Promise<IQSocketProtocolMessage<IQSocketProtocolMessageMetaAck>>;
    /**
     * @description Передача сообщения из пространства имён в соединение
     */
    private $__pipe;
    /**
     * @description Добавление слушателя события в карту событий
     */
    private addEventListener;
    /**
     * @description Создание нового сообщения в формате IQSocketProtocolChunk
     */
    private createAck;
}

declare class QSocketCompressor implements TQSocketProtocolCompressor {
    /**
     * Compresses data using the GZIP algorithm.
     * @param data Data to be compressed.
     * @returns A promise that resolves to the compressed data.
     */
    toGzip(data: Buffer | Uint8Array): Promise<Buffer | Uint8Array>;
    /**
     * Decompresses data that was compressed with the GZIP algorithm.
     * @param data Compressed data.
     * @returns A promise that resolves to the decompressed data.
     */
    fromGzip(data: Buffer | Uint8Array): Promise<Buffer | Uint8Array>;
    /**
     * Compresses data using the DEFLATE algorithm.
     * @param data Data to be compressed.
     * @returns A promise that resolves to the compressed data.
     */
    toDeflate(data: Buffer | Uint8Array): Promise<Buffer | Uint8Array>;
    /**
     * Decompresses data that was compressed with the DEFLATE algorithm.
     * @param data Compressed data.
     * @returns A promise that resolves to the decompressed data.
     */
    fromDeflate(data: Buffer | Uint8Array): Promise<Buffer | Uint8Array>;
}

export { type IQSocketProtocolConfig, QSocketClient, QSocketCompressor, QSocketConnection, QSocketDebuger, QSocketInteraction, QSocketNamespace, QSocketServer };
